@@ -163,7 +163,7 @@ corDISC <- function(phy, data, ntraits=2, rate.mat=NULL, model=c("ER","SYM","ARD
         tip.states <- lik.anc$lik.tip.states
     }else{
         TIPS <- 1:nb.tip
-        lik.anc <- ancRECON(phy, data, est.pars, rate.cat=NULL, rate.mat=rate.mat, ntraits=ntraits, method=node.states, model=model, root.p=root.p)
+        lik.anc <- ancRECON(phy, data, est.pars, rate.cat=NULL, rate.mat=model.set.final$index.matrix, ntraits=ntraits, method=node.states, model=model, root.p=root.p, collapse = FALSE)
         if(node.states == "marginal" || node.states == "scaled"){
             pr<-apply(lik.anc$lik.anc.states,1,which.max)
             phy$node.label <- pr
