@@ -6,7 +6,7 @@
 ######################################################################################################################################
 ######################################################################################################################################
 
-corHMM <- function(phy, data, rate.cat, rate.mat=NULL, model = "ARD", node.states = "marginal", fixed.nodes=FALSE, p=NULL, root.p="yang", ip=NULL, nstarts=0, n.cores=1, get.tip.states = FALSE, lewis.asc.bias = FALSE, collapse = TRUE, lower.bound = 1e-9, upper.bound = 100){
+corHMM <- function(phy, data, rate.cat, rate.mat=NULL, model = "ARD", node.states = "marginal", fixed.nodes=FALSE, p=NULL, root.p="yang", ip=NULL, nstarts=0, n.cores=1, get.tip.states = FALSE, lewis.asc.bias = FALSE, collapse = TRUE, lower.bound = 1e-9, upper.bound = 100, opts=NULL){
     
     # Checks to make sure node.states is not NULL.  If it is, just returns a diagnostic message asking for value.
     if(is.null(node.states)){
@@ -46,10 +46,16 @@ corHMM <- function(phy, data, rate.cat, rate.mat=NULL, model = "ARD", node.state
     
     nCol <- dim(data)[2]
     
-    CorData <- corProcessData(data, collapse = is.null(rate.mat))
+    CorData <- corProcessData(data, collapse = collapse)
     data.legend <- data <- CorData$corData
-    nObs <- length(CorData$ObservedTraits)
-    
+    # nObs <- length(CorData$ObservedTraits)
+    if(length(grep("&", CorData$corData[,2])) > 0){
+      non_and_chars <- as.numeric(CorData$corData[,2][-grep("&", CorData$corData[,2])])
+      and_chars <- as.numeric(unlist(strsplit(CorData$corData[,2][grep("&", CorData$corData[,2])], "&")))
+      nObs <- max(c(non_and_chars, and_chars))
+    }else{
+      nObs <- max(as.numeric(CorData$corData[,2]))
+    }
     # Checks to make sure phy & data have same taxa. Fixes conflicts (see match.tree.data function).
     matching <- match.tree.data(phy,data)
     data <- matching$data
@@ -72,8 +78,11 @@ corHMM <- function(phy, data, rate.cat, rate.mat=NULL, model = "ARD", node.state
         }
     }
     
+    if(any(phy$edge.length<=1e-5)){
+      warning("Branch lengths of 0 detected. Adding 1e-5 to these branches.", immediate. = TRUE)
+      phy$edge.length[phy$edge.length<=1e-5] <- 1e-5
+    }
     #Creates the data structure and orders the rows to match the tree.
-    phy$edge.length[phy$edge.length<=1e-5] <- 1e-5
     data.sort <- data.frame(data[,2], data[,2],row.names=data[,1])
     data.sort <- data.sort[phy$tip.label,]
     
@@ -127,8 +136,9 @@ corHMM <- function(phy, data, rate.cat, rate.mat=NULL, model = "ARD", node.state
     lower = rep(lb, model.set.final$np)
     upper = rep(ub, model.set.final$np)
     
-    
-    opts <- list("algorithm"="NLOPT_LN_SBPLX", "maxeval"="1000000", "ftol_rel"=.Machine$double.eps^0.5)
+    if(is.null(opts)){
+      opts <- list("algorithm"="NLOPT_LN_SBPLX", "maxeval"="1000000", "ftol_rel"=.Machine$double.eps^0.5)
+    }
     if(!is.null(p)){
         cat("Calculating likelihood from a set of fixed parameters", "\n")
         out<-NULL
@@ -164,7 +174,7 @@ corHMM <- function(phy, data, rate.cat, rate.mat=NULL, model = "ARD", node.state
                 if(mean.change==0){
                     starts=rep(0.01+exp(lb), model.set.final$np)
                 }else{
-                    starts<-rexp(model.set.final$np, 1/mean.change)
+                    starts<-sort(rexp(model.set.final$np, 1/mean.change), decreasing = TRUE)
                 }
                 starts[starts < exp(lb)] = exp(lb)
                 starts[starts > exp(ub)] = exp(lb)
@@ -173,7 +183,11 @@ corHMM <- function(phy, data, rate.cat, rate.mat=NULL, model = "ARD", node.state
                 tmp[,2:(model.set.final$np+1)] = out$solution
                 tmp
             }
-            restart.set<-mclapply(1:nstarts, random.restart, mc.cores=n.cores)
+            if(n.cores > 1){
+              restart.set<-mclapply(1:nstarts, random.restart, mc.cores=n.cores)
+            }else{
+              restart.set<-lapply(1:nstarts, random.restart)
+            }
             #Finds the best fit within the restart.set list
             best.fit<-which.min(unlist(lapply(restart.set, function(x) x[1])))
             #Generates an object to store results from restart algorithm:
@@ -228,6 +242,10 @@ corHMM <- function(phy, data, rate.cat, rate.mat=NULL, model = "ARD", node.state
         }
     }
     
+    if(loglik == -1e+06){
+      warning("corHMM may have failed to optimize correctly, consider checking inputs and running again.", immediate. = TRUE)
+    }
+    
     obj = list(loglik = loglik,
     AIC = AIC,
     AICc = AICc,
@@ -360,14 +378,15 @@ dev.corhmm <- function(p,phy,liks,Q,rate,root.p,rate.cat,order.test,lewis.asc.bi
           root.p = liks[root,] / sum(liks[root,])
           loglik <- -(sum(log(comp[-TIPS])) + log(sum(exp(log(root.p)+log(liks[root,])))))
       }
-  }
-  # root.p!==NULL will fix root probabilities based on user supplied vector:
-  if(is.numeric(root.p[1])){
+  }else{
+    if(is.numeric(root.p[1])){
       loglik <- -(sum(log(comp[-TIPS])) + log(sum(exp(log(root.p)+log(liks[root,])))))
       if(is.infinite(loglik)){
-          return(1000000)
+        return(1000000)
       }
+    }
   }
+  # root.p!==NULL will fix root probabilities based on user supplied vector:
   if(lewis.asc.bias == TRUE){
     p <- log(p)
     dummy.liks.vec <- getLewisLikelihood(p = p, phy = phy, liks = liks, Q = Q, rate = rate, root.p = cp_root.p, rate.cat = rate.cat)
@@ -493,7 +512,7 @@ corProcessData <- function(data, collapse=TRUE){
   if(collapse){
     corData <- data.frame(sp = data[,1], d = sapply(search.strings, function(x) paste(grep(x, ObservedTraits), collapse="&")))
   }else{
-    corData <- data.frame(sp = data[,1], d = observed.traits_index)
+    corData <- data.frame(sp = data[, 1], d = sapply(search.strings, function(x) paste(grep(x, Traits),collapse = "&")))
   }
   return(list(StateMats = StateMats,  PossibleTraits = Traits, ObservedTraits = ObservedTraits, corData = corData))
 }
@@ -508,7 +527,11 @@ print.corhmm<-function(x,...){
     cat("\n")
     
     UserStates <- corProcessData(x$data)$ObservedTraits
-    names(UserStates) <- sort(unique(x$data.legend[,2]))
+    if(length(grep("&", x$data.legend[,2])) > 0){
+      names(UserStates) <- sort(unique(as.numeric(unlist(strsplit(x$data.legend[,2], "&")))))
+    }else{
+      names(UserStates) <- sort(unique(as.numeric(x$data.legend[,2])))
+    }
     cat("Legend\n")
     print(UserStates)
     cat("\n")
@@ -530,6 +553,35 @@ print.corhmm<-function(x,...){
     }
 }
 
+# function for calculating PIR according to gardner and organ (2021)
+getPIR <- function(phy, data, collapse){
+  # calculate the CI score
+  CorData <- corProcessData(data, collapse = collapse)
+  dat <- CorData$corData
+  matching <- match.tree.data(phy,dat)
+  dat <- matching$data
+  phy <- matching$phy
+  data.sort <- data.frame(dat[,2], dat[,2],row.names=dat[,1])
+  data.sort <- data.sort[phy$tip.label,]
+  data.sort <- as.matrix(data.sort)
+  dat.phangorn <- phyDat(data.sort,type="USER", levels=1:max(as.numeric(dat[,2])))
+  phy.tmp <- multi2di(phy)
+  par.score <- parsimony(phy.tmp, dat.phangorn, method="fitch")/2
+  ci.score <- CI(phy.tmp, dat.phangorn)
+  
+  # calcualte the NIR
+  state_table <- table(dat[,2])
+  levels <- 1:max(as.numeric(dat[,2]))
+  # if all levels are observed, then we use the min from the observed states, otherwise the min is just 0
+  if(dim(state_table) == length(levels)){
+    nir.score <- (max(state_table) - min(state_table))/length(phy$tip.label)
+  }else{
+    nir.score <- (max(state_table) - 0)/length(phy$tip.label)
+  }
+  pir.score <- ci.score * nir.score 
+  return(pir.score)
+}
+
 
 ######################################################################################################################################
 ######################################################################################################################################
